﻿namespace TestLibrary.Common;

/// <summary>
/// 라이센스 키 베이스
/// </summary>
public class LincenseKeyBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Protected

    #region 키 정규화하기 - NormalizeKey(key)

    /// <summary>
    /// 키 정규화하기
    /// </summary>
    /// <param name="key">키</param>
    /// <returns>정규화 키</returns>
    protected static string NormalizeKey(string key)
    {
        if(key == null)
        {
            key = string.Empty;
        }

        return key.Trim().ToUpper().Replace("-", string.Empty).Replace(" ", string.Empty);
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Protected

    #region 키 바이트 구하기 - GetKeyByte(seed, a, b, c)

    /// <summary>
    /// 키 바이트 구하기
    /// </summary>
    /// <param name="seed">시드</param>
    /// <param name="a">바이트 A</param>
    /// <param name="b">바이트 B</param>
    /// <param name="c">바이트 C</param>
    /// <returns>키 바이트</returns>
    /// <remarks>
    /// 시드와 일부 입력 바이트가 주어지면 반환할 단일 바이트를 생성한다.
    /// 이것은 동일한 키를 검색하기 위해 나타낼 수 있는 무작위 데이터와 함께 사용해야 한다.
    /// </remarks>
    protected byte GetKeyByte(long seed, byte a, byte b, byte c)
    {
        int temporaryA = a % 25;
        int temporaryB = b % 3;

        long result;

        if((a % 2) == 0)
        {
            result = ((seed >> temporaryA) & 0xFF) ^ ((seed >> temporaryB) | c);
        }
        else
        {
            result = ((seed >> temporaryA) & 0xFF) ^ ((seed >> temporaryB) & c);
        }

        return (byte)result;
    }

    #endregion
    #region 체크썸 구하기 - GetChecksum(key)

    /// <summary>
    /// 체크썸 구하기
    /// </summary>
    /// <param name="key">키</param>
    /// <returns>체크썸</returns>
    protected string GetChecksum(string key)
    {
        ushort left  = 0x56;
        ushort right = 0xaf;

        if(key.Length > 0)
        {
            for(int count = 0; count < key.Length; count++)
            {
                right = (ushort)(right + Convert.ToByte(key[count]));

                if(right > 0xff)
                {
                    right -= 0xff;
                }

                left += right;

                if(left > 0xff)
                {
                    left -= 0xff;
                }
            }
        }

        ushort checksum = (ushort)((left << 8) + right);

        return checksum.ToString("X4");
    }

    #endregion
}