﻿using TestLibrary.Common;

namespace TestLibrary.Generator;

/// <summary>
/// 라이센스 키 제너레이터
/// </summary>
public class LicenseKeyGenerator : LincenseKeyBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 키 생성하기 - GenerateKey(seed, keyByteSetArray)

    /// <summary>
    /// 키 생성하기
    /// </summary>
    /// <param name="seed">시드</param>
    /// <param name="keyByteSetArray">키 바이트 세트 배열</param>
    /// <returns>키</returns>
    public string GenerateKey(int seed, KeyByteSet[] keyByteSetArray)
    {
        if(keyByteSetArray.Length < 2)
        {
            throw new InvalidOperationException("The KeyByteSet array must be of length 2 or greater.");
        }

        Array.Sort(keyByteSetArray, new KeyByteSetComparer());

        bool allKeyByteNumbersDistinct = true;

        List<int> keyByteNumberList = new List<int>();

        int maximumKeyByteNumber = 0;

        foreach(KeyByteSet keyByteSet in keyByteSetArray)
        {
            if(!keyByteNumberList.Contains(keyByteSet.KeyByteNumber))
            {
                keyByteNumberList.Add(keyByteSet.KeyByteNumber);

                if(keyByteSet.KeyByteNumber > maximumKeyByteNumber)
                {
                    maximumKeyByteNumber = keyByteSet.KeyByteNumber;
                }
            }
            else
            {
                allKeyByteNumbersDistinct = false;

                break;
            }
        }

        if(!allKeyByteNumbersDistinct)
        {
            throw new InvalidOperationException("The KeyByteSet array contained at least 1 item with a duplicate KeyByteNumber value.");
        }

        if(maximumKeyByteNumber != keyByteSetArray.Length)
        {
            throw new InvalidOperationException("The values for KeyByteNumber in each KeyByteSet item must be sequential and start with the number 1.");
        }

        byte[] keyByteArray = new byte[keyByteSetArray.Length];

        for(int i = 0; i < keyByteSetArray.Length; i++)
        {
            keyByteArray[i] = GetKeyByte
            (
                seed,
                keyByteSetArray[i].KeyByteA,
                keyByteSetArray[i].KeyByteB,
                keyByteSetArray[i].KeyByteC
            );
        }

        string result = seed.ToString("X8");

        for(int i = 0; i < keyByteArray.Length; i++)
        {
            result = result + keyByteArray[i].ToString("X2");
        }

        result = result + GetChecksum(result);

        int startPosition = 7;

        while(startPosition < (result.Length - 1))
        {
            result = result.Insert(startPosition, "-");

            startPosition = startPosition + 7;
        }

        return result;
    }

    #endregion
}